@@ -1,8 +1,9 @@
+from traceback import extract_tb
+from os import getcwd, path
 from sanic.request import Request
+from sanic.response import json
 from sanic.exceptions import SanicException
 from jsonclasses.exceptions import ObjectNotFoundException, ValidationException
-from sanic.response import json
-from traceback import extract_tb
 
 class UnsupportedMediaTypeException(Exception):
 
@@ -16,6 +17,9 @@ class NotAcceptableException(Exception):
     self.message = f'Accept \'{accept}\' is not supported.'
     super().__init__(self.message)
 
+def remove_none(obj: dict) -> dict:
+  return { k: v for k, v in obj.items() if v is not None }
+
 def exception_handler(request: Request, exception: Exception):
   code = exception.status_code if isinstance(exception, SanicException) else 500
   code = 415 if isinstance(exception, UnsupportedMediaTypeException) else code
@@ -23,9 +27,10 @@ def exception_handler(request: Request, exception: Exception):
   code = 404 if isinstance(exception, ObjectNotFoundException) else code
   code = 400 if isinstance(exception, ValidationException) else code
   return json({
-    'error': {
+    'error': remove_none({
       'type': exception.__class__.__name__,
-      'message': exception.message if hasattr(exception, 'message') else str(exception),
-      'traceback': [ f'file {f.filename} line {f.lineno} in {f.name}' for f in extract_tb(exception.__traceback__) ]
-    }
+      'message': str(exception),
+      'fields': exception.keypath_messages if isinstance(exception, ValidationException) else None,
+      'traceback': [ f'file {path.relpath(f.filename, getcwd())}:{f.lineno} in {f.name}' for f in extract_tb(exception.__traceback__) ],
+    })
   }, status=code)
